'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function useGetEndPoint(endpoint, keysArg, executeAjax, catchErrors) {
  // Allow effects to access the latest key values
  var keys = keysArg || {};
  var latestKeys = react.useRef(keys); // Don't submit this endpoint if it is not enabled
  // because of `!== false` it is set to true by default

  var isEndpointEnabled = executeAjax !== false; // Promise to wait for executeAjax to be true for the first time
  // This is neccessary to postpone the initial ajax execution

  var _useWaitForValuePromi = useWaitForValuePromise(isEndpointEnabled, function (enabled) {
    return enabled;
  }),
      endpointBecomesEnabledPromise = _useWaitForValuePromi[0],
      endpointEnabledState = _useWaitForValuePromi[1]; // Calculate the cacheKey only if the keys change
  // empty keys are threaten as null to make sure as
  // in javascript `{} !== {}` but `null === null`


  var cacheKey = react.useMemo(function () {
    return endpoint.getCacheKey(keys);
  }, [Object.keys(keys).length ? keys : null]);
  latestKeys.current = keys; // Helper to start ajax loading

  var executeEndpoint = react.useCallback(function () {
    // Once the endpoint is enabeld return the endpoint(latestKeys.current)
    // from cache to prevent endless loops during suspense
    var currendEndpointPromise = endpointEnabledState.done ? endpoint(latestKeys.current) : endpointBecomesEnabledPromise.then(function () {
      return endpoint(latestKeys.current);
    }); // Once data is received update the internal hook state

    currendEndpointPromise.then(function (result) {
      updateEndpointState({
        type: "receiveData",
        promise: currendEndpointPromise,
        value: result
      });
    }, function (error) {
      updateEndpointState({
        type: "receiveError",
        promise: currendEndpointPromise,
        value: error
      });

      if (!catchErrors) {
        return Promise.reject(error);
      }
    });
    return currendEndpointPromise;
  }, []);

  var _useEndpointStateRedu = useEndpointStateReduce(executeEndpoint),
      endpointState = _useEndpointStateRedu[0],
      updateEndpointState = _useEndpointStateRedu[1]; // Whenever the keys change execute the endpoints again


  react.useEffect(function () {
    if (!isEndpointEnabled) {
      return;
    } // Track this hook as endpoint consumer
    // once all consumers are gone the memory will be freed


    return endpoint.observePromise(latestKeys.current, function () {
      var result = executeEndpoint();
      updateEndpointState({
        type: "executeAjax",
        promise: executeEndpoint()
      });
      return result;
    });
  }, [endpoint, cacheKey, isEndpointEnabled]);
  return endpointState;
} // Suspense will unmount the component while it is loading
// Therefore the promise result is storred in a WeakMap instead
// of a useRef, useState or useReduce helper

var suspenseCache = /*#__PURE__*/new WeakMap();
/**
 * Provide a hook which works with React.Suspense
 */

function useGetEndPointSuspendable(endpoint, keys) {
  var endpointData = useGetEndPoint(endpoint, keys);

  if (endpointData.state === "ERROR") {
    throw new Error("Endpoint failed");
  }

  if (endpointData.state === "LOADING" || endpointData.state === "UPDATING") {
    // Use the suspense cache to detect if the promise is already loaded
    // this works only because endpointData.promise wil get the same promise
    // from the request-registry cache
    var suspenseCacheEntry = suspenseCache.get(endpointData.promise) || {
      promise: endpointData.promise.then(function (value) {
        suspenseCacheEntry.value = value;
      }, function (error) {
        suspenseCacheEntry.error = error;
      }).then(function () {
        return endpointData.promise;
      })
    };
    suspenseCache.set(endpointData.promise, suspenseCacheEntry); // Check if the suspense cache could retrieve a value or an error

    if (suspenseCacheEntry.value) {
      return suspenseCacheEntry.value;
    }

    if (suspenseCacheEntry.error) {
      throw new Error("Endpoint failed");
    } // If the value and error aren't loaded
    // throw the promise to tell react to rerender once the
    // suspense cache has data


    throw suspenseCacheEntry.promise;
  }

  return endpointData.value;
}
function useGetEndPointLazy(endpoint, keys, executeAjax) {
  var serverSideState = {
    busy: true,
    value: undefined,
    state: "LOADING",
    hasData: false,
    error: undefined,
    promise: new Promise(function () {})
  };
  return typeof window === "undefined" ? serverSideState : useGetEndPoint(endpoint, keys, executeAjax);
}

var initialEndpointState = function initialEndpointState(initialLoad) {
  return {
    busy: true,
    value: undefined,
    state: "LOADING",
    hasData: false,
    error: undefined,
    promise: initialLoad()
  };
};

function endpointStateReducer(state, action) {
  if (action.type === "executeAjax") {
    if (!state.hasData) return {
      promise: action.promise,
      state: "LOADING",
      busy: true,
      error: undefined,
      hasData: false,
      value: undefined
    };
    return {
      promise: action.promise,
      state: "UPDATING",
      busy: true,
      hasData: true,
      error: state.error,
      value: state.value
    };
  } // Keep the same state if the promise is not upto date anymore


  if (state.promise !== action.promise) {
    return state;
  }

  switch (action.type) {
    case "receiveData":
      return _extends({}, state, {
        state: "DONE",
        busy: false,
        hasData: true,
        error: undefined,
        value: action.value
      });

    case "receiveError":
      return _extends({}, state, {
        state: "ERROR",
        busy: false,
        hasData: false,
        error: action.value,
        value: undefined
      });

    /* istanbul ignore next */

    default:
      var unhandledAction = action;
      throw String(unhandledAction);
  }
}

function useEndpointStateReduce(initialLoad) {
  return react.useReducer(endpointStateReducer, initialLoad, initialEndpointState);
}

function useWaitForValuePromise(value, conditionFunction) {
  var _useMemo = react.useMemo(function () {
    var refObject = {
      done: conditionFunction(value)
    };
    var innerResolve = undefined;
    var promise = new Promise(function (resolve) {
      innerResolve = resolve;
    });

    if (refObject.done) {
      return [Promise.resolve(value), innerResolve, refObject];
    }

    return [promise, innerResolve, refObject];
  }, []),
      promise = _useMemo[0],
      resolve = _useMemo[1],
      ref = _useMemo[2];

  react.useEffect(function () {
    if (resolve && !ref.done) {
      ref.done = conditionFunction(value);

      if (ref.done) {
        resolve(value);
      }
    }
  }, [value]);
  return [promise, ref];
}

exports.useGetEndPoint = useGetEndPoint;
exports.useGetEndPointLazy = useGetEndPointLazy;
exports.useGetEndPointSuspendable = useGetEndPointSuspendable;
//# sourceMappingURL=request-registry-react.cjs.development.js.map
